@@ -16,14 +16,29 @@ TABLE_FOOTER = "TABLE_FOOTER"
 TABLE_SUMMARY = "TABLE_SUMMARY"
 TABLE_SECTION_TITLE = "TABLE_SECTION_TITLE"
 TABLE_COLUMN_HEADER = "COLUMN_HEADER"
-TABLE_STRUCTURED = "STRUCTURED"
-TABLE_SEMI_STRUCTURED = "SEMI_STRUCTURED"
+TABLE_STRUCTURED = "STRUCTURED_TABLE"
+TABLE_SEMI_STRUCTURED = "SEMI_STRUCTURED_TABLE"
 CELL = "CELL"
 PAGE = "PAGE"
 MERGED_CELL = "MERGED_CELL"
 QUERY = "QUERY"
 QUERY_RESULT = "QUERY_RESULT"
 SIGNATURE = "SIGNATURE"
+LAYOUT = "LAYOUT"
+LAYOUT_TEXT = "LAYOUT_TEXT"
+LAYOUT_TITLE = "LAYOUT_TITLE"
+LAYOUT_HEADER = "LAYOUT_HEADER"
+LAYOUT_FOOTER = "LAYOUT_FOOTER"
+LAYOUT_SECTION_HEADER = "LAYOUT_SECTION_HEADER"
+LAYOUT_PAGE_NUMBER = "LAYOUT_PAGE_NUMBER"
+LAYOUT_LIST = "LAYOUT_LIST"
+LAYOUT_FIGURE = "LAYOUT_FIGURE"
+LAYOUT_TABLE = "LAYOUT_TABLE"
+LAYOUT_KEY_VALUE = "LAYOUT_KEY_VALUE"
+
+# This is not a base layout type, but a fake type for the text conversion logic
+# when an API is called without the AnalyzeLayout API, all the elements will be LAYOUT_ENTITY
+LAYOUT_ENTITY = "LAYOUT_ENTITY"
 
 # cell type attributes
 IS_COLUMN_HEAD = "isColumnHead"
@@ -37,6 +52,7 @@ IS_MERGED_CELL = "isMergedCell"
 HANDWRITING = "HANDWRITING"
 PRINTED = "PRINTED"
 
+
 class TableTypes(Enum):
     """Types of tables recognized by Textract APIs."""
 
@@ -44,6 +60,7 @@ class TableTypes(Enum):
     STRUCTURED = 1
     SEMI_STRUCTURED = 2
 
+
 class Direction(Enum):
     """Directions available for search using DirectionalFinder"""
 
@@ -97,6 +114,7 @@ DEFAULT = "DEFAULT"
 FORMS = "FORMS"
 TABLES = "TABLES"
 QUERIES = "QUERIES"
+LAYOUT = "LAYOUT"
 
 
 class TextractFeatures(Enum):
@@ -106,6 +124,7 @@ class TextractFeatures(Enum):
     TABLES = 1
     QUERIES = 2
     SIGNATURES = 3
+    LAYOUT = 4
 
 
 class TextractType(Enum):
@@ -224,6 +243,7 @@ class AnalyzeIDFields(Enum):
     # Only available in passports
     PLACE_OF_BIRTH = "PLACE_OF_BIRTH"
 
+
 class AnalyzeExpenseLineItemFields(Enum):
     ITEM = "ITEM"
     PRICE = "PRICE"
@@ -232,6 +252,7 @@ class AnalyzeExpenseLineItemFields(Enum):
     UNIT_PRICE = "UNIT_PRICE"
     EXPENSE_ROW = "EXPENSE_ROW"
 
+
 class AnalyzeExpenseFields(Enum):
     ACCOUNT_NUMBER = "ACCOUNT_NUMBER"
     ADDRESS = "ADDRESS"
@@ -278,6 +299,7 @@ class AnalyzeExpenseFields(Enum):
     VENDOR_VAT_NUMBER = "VENDOR_VAT_NUMBER"
     ZIP_CODE = "ZIP_CODE"
 
+
 class AnalyzeExpenseFieldsGroup(Enum):
     RECEIVER = "RECEIVER"
     RECEIVER_BILL_TO = "RECEIVER_BILL_TO"
@@ -287,6 +309,7 @@ class AnalyzeExpenseFieldsGroup(Enum):
     VENDOR_REMIT_TO = "VENDOR_REMIT_TO"
     VENDOR_SUPPLIER = "VENDOR_SUPPLIER"
 
+
 class CLIPrint(Enum):
     ALL = 0
     TEXT = 1
@@ -296,7 +319,7 @@ class CLIPrint(Enum):
     EXPENSES = 5
     SIGNATURES = 6
     IDS = 7
-
+    LAYOUTS = 8
 
 class CLIOverlay(Enum):
     ALL = 0
@@ -305,4 +328,5 @@ class CLIOverlay(Enum):
     TABLES = 3
     FORMS = 4
     QUERIES = 5
-    SIGNATURE = 6
+    SIGNATURES = 6
+    LAYOUTS = 7
