@@ -8,13 +8,12 @@ import string
 import logging
 import xlsxwriter
 import io
-from typing import List, IO, Union, AnyStr
+from pathlib import Path
+from typing import List, IO, Union, AnyStr, Tuple, Optional
 from copy import deepcopy
 from collections import defaultdict
 from PIL import Image
 
-from trp.trp2 import TDocument, TDocumentSchema
-
 from textractor.entities.expense_document import ExpenseDocument
 from textractor.entities.identity_document import IdentityDocument
 from textractor.entities.word import Word
@@ -23,10 +22,10 @@ from textractor.entities.page import Page
 from textractor.entities.table import Table
 from textractor.entities.query import Query
 from textractor.entities.signature import Signature
-from textractor.exceptions import InputError
+from textractor.entities.layout import Layout
+from textractor.exceptions import InputError, MissingDependencyException
 from textractor.entities.key_value import KeyValue
 from textractor.entities.bbox import SpatialObject
-from textractor.data.constants import SelectionStatus
 from textractor.utils.s3_utils import download_from_s3
 from textractor.visualizers.entitylist import EntityList
 from textractor.data.constants import (
@@ -35,11 +34,14 @@ from textractor.data.constants import (
     Direction,
     DirectionalFinderType,
 )
-from textractor.entities.selection_element import SelectionElement
-from textractor.utils.search_utils import SearchUtils, jaccard_similarity
+from textractor.utils.search_utils import SearchUtils
+from textractor.data.text_linearization_config import TextLinearizationConfig
+from textractor.data.html_linearization_config import HTMLLinearizationConfig
+from textractor.entities.linearizable import Linearizable
 
+logger = logging.getLogger(__name__)
 
-class Document(SpatialObject):
+class Document(SpatialObject, Linearizable):
     """
     Represents the description of a single document, as it would appear in the input to the Textract API.
     Document serves as the root node of the object model hierarchy,
@@ -48,12 +50,12 @@ class Document(SpatialObject):
     """
 
     @classmethod
-    def open(cls, fp: Union[dict, str, IO[AnyStr]]):
+    def open(cls, fp: Union[dict, str, Path, IO[AnyStr]]):
         """Create a Document object from a JSON file path, file handle or response dictionary
 
         :param fp: _description_
-        :type fp: Union[dict, str, IO[AnyStr]]
-        :raises InputError: Raised on input not being of type Union[dict, str, IO[AnyStr]]
+        :type fp: Union[dict, str, Path, IO[AnyStr]]
+        :raises InputError: Raised on input not being of type Union[dict, str, Path, IO[AnyStr]]
         :return: Document object
         :rtype: Document
         """
@@ -63,16 +65,19 @@ class Document(SpatialObject):
             return response_parser.parse(fp)
         elif isinstance(fp, str):
             if fp.startswith("s3://"):
-                # FIXME: Opening s3 clients for everythign should be avoided
+                # FIXME: Opening s3 clients for everything should be avoided
                 client = boto3.client("s3")
                 return response_parser.parse(json.load(download_from_s3(client, fp)))
             with open(fp, "r") as f:
                 return response_parser.parse(json.load(f))
+        elif isinstance(fp, Path):
+            with open(fp, "r") as f:
+                return response_parser.parse(json.load(f))
         elif isinstance(fp, io.IOBase):
             return response_parser.parse(json.load(fp))
         else:
             raise InputError(
-                f"Document.open() input must be of type dict, str or file handle, not {type(fp)}"
+                f"Document.open() input must be of type dict, str, Path or a file handle, not {type(fp)}"
             )
 
     def __init__(self, num_pages: int = 1):
@@ -145,7 +150,7 @@ class Document(SpatialObject):
         return EntityList(sum([page.lines for page in self.pages], []))
 
     @property
-    def key_values(self) -> List[KeyValue]:
+    def key_values(self) -> EntityList[KeyValue]:
         """
         Returns all the :class:`KeyValue` objects present in the Document.
 
@@ -155,7 +160,7 @@ class Document(SpatialObject):
         return EntityList(sum([page.key_values for page in self.pages], []))
 
     @property
-    def checkboxes(self) -> List[KeyValue]:
+    def checkboxes(self) -> EntityList[KeyValue]:
         """
         Returns all the :class:`KeyValue` objects with SelectionElements present in the Document.
 
@@ -165,7 +170,7 @@ class Document(SpatialObject):
         return EntityList(sum([page.checkboxes for page in self.pages], []))
 
     @property
-    def tables(self) -> List[Table]:
+    def tables(self) -> EntityList[Table]:
         """
         Returns all the :class:`Table` objects present in the Document.
 
@@ -175,7 +180,7 @@ class Document(SpatialObject):
         return EntityList(sum([page.tables for page in self.pages], []))
 
     @property
-    def queries(self) -> List[Query]:
+    def queries(self) -> EntityList[Query]:
         """
         Returns all the :class:`Query` objects present in the Document.
 
@@ -185,7 +190,7 @@ class Document(SpatialObject):
         return EntityList(sum([page.queries for page in self.pages], []))
 
     @property
-    def signatures(self) -> List[Signature]:
+    def signatures(self) -> EntityList[Signature]:
         """
         Returns all the :class:`Signature` objects present in the Document.
 
@@ -194,6 +199,16 @@ class Document(SpatialObject):
         """
         return EntityList(sum([page.signatures for page in self.pages], []))
 
+    @property
+    def layouts(self) -> EntityList[Layout]:
+        """
+        Returns all the :class:`Layout` objects present in the Document
+
+        :return: List of Layout objects
+        :rtype: EntityList[Layout]
+        """
+        return EntityList(sum([page.layouts for page in self.pages], []))
+
     @property
     def identity_document(self) -> EntityList[IdentityDocument]:
         """
@@ -245,6 +260,15 @@ class Document(SpatialObject):
         """
         self._pages = sorted(pages, key=lambda x: x.page_num)
 
+    def get_text_and_words(
+        self, config: TextLinearizationConfig = TextLinearizationConfig()
+    ) -> Tuple[str, List]:
+        text, words_lists = zip(*[p.get_text_and_words(config) for p in self.pages])
+        flattened_words = []
+        for words in words_lists:
+            flattened_words.extend(words)
+        return config.layout_element_separator.join(text), flattened_words
+
     def page(self, page_no: int = 0):
         """
         Returns :class:`Page` object/s depending on the input page_no. Follows zero-indexing.
@@ -263,6 +287,22 @@ class Document(SpatialObject):
         else:
             raise InputError("page_no parameter doesn't match required data type.")
 
+    def to_html(self, config: HTMLLinearizationConfig = HTMLLinearizationConfig()):
+        """
+        Returns the HTML representation of the document, effectively calls Linearizable.to_html()
+        but add <html><body></body></html> around the result and put each page in a <div>. 
+
+        :return: HTML text of the entity
+        :rtype: str
+        """
+        
+        html = "<html><body>"
+        for page in self.pages:
+            html += f"<div>{page.to_html(config=config)}</div>"
+        html += "</body></html>"
+        
+        return html
+
     def __repr__(self):
         return os.linesep.join(
             [
@@ -280,18 +320,26 @@ class Document(SpatialObject):
             ]
         )
 
-    def to_trp2(self) -> TDocument:
+    def to_trp2(self):
         """
         Parses the response to the trp2 format for backward compatibility
 
         :return: TDocument object that can be used with the older Textractor libraries
         :rtype: TDocument
         """
+        from trp.trp2 import TDocument, TDocumentSchema
+        
         if not self._trp2_document:
             self._trp2_document = TDocumentSchema().load(self.response)
         return self._trp2_document
 
     def visualize(self, *args, **kwargs):
+        """
+        Returns the object's children in a visualization EntityList object
+
+        :return: Returns an EntityList object
+        :rtype: EntityList
+        """
         return EntityList(self.pages).visualize(*args, **kwargs)
 
     def keys(self, include_checkboxes: bool = True) -> List[str]:
@@ -342,7 +390,7 @@ class Document(SpatialObject):
         :rtype: EntityList[Word]
         """
         if not self.words:
-            logging.warn("Document contains no word entities.")
+            logger.warning("Document contains no word entities.")
             return []
 
         filtered_words = EntityList()
@@ -507,12 +555,12 @@ class Document(SpatialObject):
                 lowest_similarity = top_n[-1][1]
 
         if not top_n:
-            logging.warning(
+            logger.warning(
                 f"Query key does not match any existing keys in the document.{os.linesep}{self.keys()}"
             )
             return EntityList([])
 
-        logging.info(f"Query key matched {len(top_n)} key-values in the document.")
+        logger.info(f"Query key matched {len(top_n)} key-values in the document.")
 
         return EntityList([value[0] for value in top_n])
 
@@ -522,6 +570,7 @@ class Document(SpatialObject):
         include_kv: bool = True,
         include_checkboxes: bool = True,
         filepath: str = "Key-Values.csv",
+        sep: str = ";",
     ):
         """
         Export key-value entities and checkboxes in csv format.
@@ -532,32 +581,77 @@ class Document(SpatialObject):
         :type include_checkboxes: bool
         :param filepath: Path to where file is to be stored.
         :type filepath: str
+        :param sep: Separator to be used in the csv file.
+        :type sep: str
         """
         keys = []
         values = []
         if include_kv and not self.key_values:
-            logging.warning("Document does not contain key-values.")
+            logger.warning("Document does not contain key-values.")
         elif include_kv:
             for kv in self.key_values:
-                keys.append(kv.key.__repr__())
-                values.append(kv.value.__repr__())
+                keys.append(" ".join([w.text for w in kv.key]))
+                values.append(kv.value.get_text())
 
         if include_checkboxes and not self.checkboxes:
-            logging.warning("Document does not contain checkbox elements.")
+            logger.warning("Document does not contain checkbox elements.")
         elif include_checkboxes:
             for kv in self.checkboxes:
-                keys.append(kv.key.__repr__())
+                keys.append(" ".join([w.text for w in kv.key]))
                 values.append(kv.value.children[0].status.name)
 
         with open(filepath, "w") as f:
-            f.write(f"Key,Value{os.linesep}")
+            f.write(f"Key{sep}Value{os.linesep}")
             for k, v in zip(keys, values):
-                f.write(f"{k},{v}{os.linesep}")
+                f.write(f"{k}{sep}{v}{os.linesep}")
 
-        logging.info(
+        logger.info(
             f"csv file stored at location {os.path.join(os.getcwd(),filepath)}"
         )
 
+    def export_kv_to_pandas(
+        self, 
+        trim: bool = True, 
+        confidence_threshold: float = 0.0, 
+        drop_confidence: bool = True
+        ):
+        """
+        Converts key-value pairs with optional confidence filtering into a pandas DataFrame.
+
+        :param trim: Flag to trim whitespace and punctuation from key and value text. Default is True.
+        :type trim: bool
+        :param confidence_threshold: Minimum confidence level required for a key-value pair to be included in the DataFrame. Default is 0.0.
+        :type confidence_threshold: float
+        :param drop_confidence: Flag to exclude the confidence column from the DataFrame. Default is True.
+        :type drop_confidence: bool
+
+        :return: A pandas DataFrame containing key-value pairs, and optionally their confidence scores.
+                The DataFrame will have 'Key' and 'Value' columns, and a 'Confidence' column if `drop_confidence` is False.
+        :rtype: pd.DataFrame
+        """
+        try:
+            from pandas import DataFrame
+        except ImportError:
+            raise MissingDependencyException("The pandas library is required for exporting tables to DataFrame objects. Please install it with `pip install pandas`.")
+        
+        keys: list[str] = []
+        values: list[str] = []
+        confidences: Optional[list[float]] = [] if not drop_confidence else None
+        
+        # Loop through key-values and filter by confidence threshold
+        for row in self.key_values:
+            if row.confidence > confidence_threshold:
+                keys.append(row.key.text.strip(": ").strip() if trim else row.key.text)
+                values.append(row.value.text.strip() if trim else row.value.text)
+                if confidences is not None:
+                    confidences.append(row.confidence)
+
+        data = {'Key': keys, 'Value': values}
+        if confidences is not None:
+            data['Confidence'] = confidences
+
+        return DataFrame(data)
+
     def export_kv_to_txt(
         self,
         include_kv: bool = True,
@@ -577,7 +671,7 @@ class Document(SpatialObject):
         export_str = ""
         index = 1
         if include_kv and not self.key_values:
-            logging.warning("Document does not contain key-values.")
+            logger.warning("Document does not contain key-values.")
         elif include_kv:
             for kv in self.key_values:
                 export_str += (
@@ -586,7 +680,7 @@ class Document(SpatialObject):
                 index += 1
 
         if include_checkboxes and not self.checkboxes:
-            logging.warning("Document does not contain checkbox elements.")
+            logger.warning("Document does not contain checkbox elements.")
         elif include_checkboxes:
             for kv in self.checkboxes:
                 export_str += f"{index}. {kv.key.__repr__()} : {kv.value.children[0].status.name}{os.linesep}"
@@ -594,7 +688,7 @@ class Document(SpatialObject):
 
         with open(filepath, "w") as text_file:
             text_file.write(export_str)
-        logging.info(
+        logger.info(
             f"txt file stored at location {os.path.join(os.getcwd(),filepath)}"
         )
 
@@ -607,7 +701,7 @@ class Document(SpatialObject):
         :type filepath: str, required
         """
         if not filepath:
-            logging.error("Filepath required to store excel file.")
+            logger.error("Filepath required to store excel file.")
         workbook = xlsxwriter.Workbook(filepath)
         for table in self.tables:
             workbook = table.to_excel(
@@ -621,7 +715,7 @@ class Document(SpatialObject):
         :rtype: EntityList[Word]
         """
         if not self.words:
-            logging.warning("Words have not been assigned to this Document object.")
+            logger.warning("Words have not been assigned to this Document object.")
             return []
 
         else:
@@ -704,7 +798,7 @@ class Document(SpatialObject):
         final_kv = []
         for kv in new_key_values:
             if kv.key:
-                key_words = [deepcopy(word) for word in kv.key.words]
+                key_words = [deepcopy(word) for word in kv.key]
                 key_words[0].text = prefix + key_words[0].text
                 new_kv = deepcopy(kv)
                 new_kv.key = key_words
@@ -774,7 +868,7 @@ class Document(SpatialObject):
         )
 
         if not word_1_objects:
-            logging.warning(f"{word_1} not found in page {page}")
+            logger.warning(f"{word_1} not found in page {page}")
             return -1, -1, -1, -1
         else:
             word_1_obj = word_1_objects[0]
@@ -789,7 +883,7 @@ class Document(SpatialObject):
             )
             word_2_objects = [word for word in word_2_objects if word.page == page]
             if not word_2_objects:
-                logging.warning(f"{word_2} not found in page {page}")
+                logger.warning(f"{word_2} not found in page {page}")
                 return -1, -1, -1, -1
             else:
                 word_2_obj = word_2_objects[0]
