@@ -8,16 +8,19 @@ bounding box information, value, existence of checkbox, page number, Page ID and
 
 import logging
 from typing import List
+import uuid
 
-from textractor.entities.line import Line
 from textractor.entities.word import Word
 from textractor.entities.value import Value
 from textractor.exceptions import InputError
 from textractor.entities.bbox import BoundingBox
 from textractor.entities.document_entity import DocumentEntity
 from textractor.data.constants import TextTypes
+from textractor.data.text_linearization_config import TextLinearizationConfig
 from textractor.visualizers.entitylist import EntityList
+from textractor.utils.html_utils import add_id_to_html_tag
 
+logger = logging.getLogger(__name__)
 
 class KeyValue(DocumentEntity):
     """
@@ -45,11 +48,11 @@ class KeyValue(DocumentEntity):
     ):
         super().__init__(entity_id, bbox)
 
-        self._words: List[Word] = []
+        self._words: EntityList[Word] = []
         self.contains_checkbox = contains_checkbox
+        self._confidence = confidence / 100
         self._value: Value = value
         self.selection_status = False
-        self.confidence = confidence / 100
         self._page = None
         self._page_id = None
 
@@ -59,15 +62,25 @@ class KeyValue(DocumentEntity):
         Return the average OCR confidence
         :return:
         """
-        key_confidences = [w.confidence for w in self.key.words]
-        value_confidences = [w.confidence for w in self.value.words] if self.value else []
+        key_confidences = [w.confidence for w in self.key]
+        value_confidences = (
+            [w.confidence for w in self.value.words] if self.value else []
+        )
 
-        key_confidence = sum(key_confidences) / len(key_confidences) if key_confidences else None
-        value_confidence = sum(value_confidences) / len(value_confidences) if value_confidences else None
-        return {"key_average": key_confidence,
-                "key_min": min(key_confidences) if key_confidences else None,
-                "value_average": value_confidence,
-                "value_min": min(value_confidences) if value_confidences else None}
+        key_confidence = (
+            sum(key_confidences) / len(key_confidences) if key_confidences else None
+        )
+        value_confidence = (
+            sum(value_confidences) / len(value_confidences)
+            if value_confidences
+            else None
+        )
+        return {
+            "key_average": key_confidence,
+            "key_min": min(key_confidences) if key_confidences else None,
+            "value_average": value_confidence,
+            "value_min": min(value_confidences) if value_confidences else None,
+        }
 
     @property
     def words(self) -> List[Word]:
@@ -77,7 +90,7 @@ class KeyValue(DocumentEntity):
         :return words: List of Word objects, each representing a word within the KeyValue entity.
         :rtype: EntityList[Word]
         """
-        value_words = self.value.words if not self.contains_checkbox else []
+        value_words = self.value.words if self.value is not None and not self.contains_checkbox else []
         return EntityList(
             sorted(self._words + value_words, key=lambda x: x.bbox.x + x.bbox.y)
         )
@@ -85,19 +98,13 @@ class KeyValue(DocumentEntity):
     @property
     def key(self):
         """
-        :return: Returns :class:`Line` object associated with the key.
-        :rtype: Line
+        :return: Returns :class:`EntityList[Word]` object (a list of words) associated with the key.
+        :rtype: EntityList[Word]
         """
         if not self._words:
-            logging.warning(f"Key contains no words objects.")
+            logger.info("Key contains no words objects.")
             return []
-        else:
-            return Line(
-                entity_id=self.id,
-                bbox=self.bbox,
-                words=self._words,
-                confidence=self.confidence * 100,
-            )
+        return self._words
 
     @key.setter
     def key(self, words: List[Word]):
@@ -108,7 +115,7 @@ class KeyValue(DocumentEntity):
         No specific ordering is assumed.
         :type words: list
         """
-        self._words = sorted(words, key=lambda x: x.bbox.x + x.bbox.y)
+        self._words = EntityList(words)
 
     @property
     def value(self) -> Value:
@@ -117,9 +124,9 @@ class KeyValue(DocumentEntity):
         :rtype: Value
         """
         if self._value is None:
-            logging.warning(
-                f"Asked for a value but it was never attributed "
-                f"-> make sure to assign value to key with the `kv.value = <Value Object>` property setter"
+            logger.warning(
+                "Asked for a value but it was never attributed "
+                "-> make sure to assign value to key with the `kv.value = <Value Object>` property setter"
             )
             return None
         else:
@@ -187,7 +194,7 @@ class KeyValue(DocumentEntity):
             )
 
         if not self.words:
-            logging.info("Document contains no word entities.")
+            logger.info("Document contains no word entities.")
             return []
         else:
             return EntityList(
@@ -195,20 +202,65 @@ class KeyValue(DocumentEntity):
             )
 
     def is_selected(self) -> bool:
+        """
+        For KeyValues containing a selection item, returns its `is_selected` status
+
+        :return: Selection status of a selection item key value pair
+        :rtype: bool
+        """
         if self.contains_checkbox:
             if len(self.value.children) == 1:
                 return self.value.children[0].is_selected()
             else:
-                logging.info(
+                logger.info(
                     "is_checked() was called on a KeyValue that contains more than one checkbox. Returning first checkbox"
                 )
                 return self.value.children[0].is_selected()
         else:
-            logging.info(
+            logger.info(
                 "is_checked() was called on a KeyValue that does not contain checkboxes. Returning False"
             )
             return False
 
+    def get_text_and_words(
+        self, config: TextLinearizationConfig = TextLinearizationConfig()
+    ):
+        key_text, key_words = " ".join([w.text for w in self.key]), self.key
+        value_text, value_words = self.value.get_text_and_words(config) if self.value else ("", "")
+        words = []
+        if not len(key_text) and not len(value_text):
+            return "", []
+        s = config.key_suffix.strip()
+        key_suffix = (
+            config.key_suffix
+            if not (len(key_text) > len(s) and key_text[-len(s) :] == s)
+            else " "
+        )
+        if config.add_prefixes_and_suffixes_in_text:
+            text = f"{add_id_to_html_tag(config.key_value_prefix, self.id, config)}{config.key_prefix}{key_text}{key_suffix}{value_text}{config.key_value_suffix}"
+        else:
+            text = f"{key_text}{config.same_paragraph_separator}{value_text}"
+
+        if config.add_prefixes_and_suffixes_as_words:
+            words += [Word(str(uuid.uuid4()), self.bbox, add_id_to_html_tag(config.key_value_prefix, self.id, config), is_structure=True)] if config.key_value_prefix else []
+            if key_words:
+                words += (
+                    ([Word(str(uuid.uuid4()), BoundingBox.enclosing_bbox(self.key), config.key_prefix, is_structure=True)] if config.key_prefix else []) +
+                    key_words +
+                    ([Word(str(uuid.uuid4()), BoundingBox.enclosing_bbox(self.key), config.key_suffix, is_structure=True)] if config.key_suffix else [])
+                )
+            if value_words: 
+                words += value_words
+            words += ([Word(str(uuid.uuid4()), self.bbox, config.key_value_suffix, is_structure=True)] if config.key_value_suffix else [])
+        else:
+            words += key_words + value_words
+
+        for w in words:
+            w.kv_id = str(self.id)
+            w.kv_bbox = self.bbox
+
+        return text, words
+
     def __repr__(self) -> str:
         """
         :return: Returns KeyValue object as a formatted string.
