@@ -16,6 +16,8 @@ from textractor.entities.word import Word
 from textractor.entities.line import Line
 from textractor.entities.table import Table
 from textractor.entities.signature import Signature
+from textractor.entities.layout import Layout
+from textractor.entities.page_layout import PageLayout
 from textractor.exceptions import InputError
 from textractor.entities.key_value import KeyValue
 from textractor.entities.query import Query
@@ -23,13 +25,28 @@ from textractor.entities.identity_document import IdentityDocument
 from textractor.entities.bbox import SpatialObject
 from textractor.data.constants import SelectionStatus, Direction, DirectionalFinderType
 from textractor.data.constants import TextTypes, SimilarityMetric
+from textractor.data.constants import (
+    LAYOUT_TEXT,
+    LAYOUT_TITLE,
+    LAYOUT_HEADER,
+    LAYOUT_FOOTER,
+    LAYOUT_SECTION_HEADER,
+    LAYOUT_PAGE_NUMBER,
+    LAYOUT_LIST,
+    LAYOUT_FIGURE,
+    LAYOUT_TABLE,
+    LAYOUT_KEY_VALUE,
+)
+from textractor.data.text_linearization_config import TextLinearizationConfig
 from textractor.entities.selection_element import SelectionElement
 from textractor.utils.geometry_util import sort_by_position
 from textractor.utils.search_utils import SearchUtils, jaccard_similarity
 from textractor.visualizers.entitylist import EntityList
+from textractor.entities.linearizable import Linearizable
 
+logger = logging.getLogger(__name__)
 
-class Page(SpatialObject):
+class Page(SpatialObject, Linearizable):
     """
     Creates a new document, ideally representing a single item in the dataset.
 
@@ -57,6 +74,7 @@ class Page(SpatialObject):
         self._tables: EntityList[Table] = EntityList([])
         self._queries: EntityList[Query] = EntityList([])
         self._expense_documents: EntityList[ExpenseDocument] = EntityList([])
+        self._layouts: EntityList[Layout] = EntityList([])
         self.kv_cache = defaultdict(list)
         self.metadata = {}
         self.page_num = page_num
@@ -107,12 +125,87 @@ class Page(SpatialObject):
 
     @property
     def text(self) -> str:
-        """Returns the page text as one string
+        """
+        Returns the page text
 
-        :return: Line text seperated by line return
+        :return: Linearized page text
         :rtype: str
         """
-        return os.linesep.join([line.text for line in self.lines])
+        return self.get_text()
+
+    def get_text_and_words(
+        self, config: TextLinearizationConfig = TextLinearizationConfig()
+    ) -> Tuple[str, List[Word]]:
+        """
+        Returns the page text and words sorted in reading order
+
+        :param config: Text linearization configuration object, defaults to TextLinearizationConfig()
+        :type config: TextLinearizationConfig, optional
+        :return: Tuple of page text and words
+        :rtype: Tuple[str, List[Word]]
+        """
+        unsorted_layouts = [l for l in self.layouts if l.reading_order < 0]
+        sorted_layouts = [l for l in self.layouts if l.reading_order >= 0]
+        if unsorted_layouts:
+            for unsorted_layout in sorted(
+                unsorted_layouts, key=lambda x: (x.bbox.y, x.bbox.x)
+            ):
+                closest_layout = None
+                closest_reading_order_distance = None
+                for layout in sorted(sorted_layouts, key=lambda x: x.reading_order):
+                    dist = layout.bbox.get_distance(unsorted_layout)
+                    if (
+                        closest_reading_order_distance is None
+                        or dist < closest_reading_order_distance
+                    ):
+                        closest_layout = layout
+                if closest_layout:
+                    sorted_layouts.insert(
+                        sorted_layouts.index(closest_layout) + 1, unsorted_layout
+                    )
+                else:
+                    sorted_layouts.append(unsorted_layout)
+
+        page_texts_and_words = [l.get_text_and_words(config) for l in sorted_layouts]
+
+        if not page_texts_and_words:
+            return "", []
+
+        text, words = zip(*page_texts_and_words)
+        combined_words = []
+        for w in words:
+            combined_words += w
+        return config.layout_element_separator.join(text), combined_words
+
+    @property
+    def page_layout(self) -> PageLayout:
+        return PageLayout(
+            titles=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_TITLE]
+            ),
+            headers=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_HEADER]
+            ),
+            footers=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_FOOTER]
+            ),
+            section_headers=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_SECTION_HEADER]
+            ),
+            page_numbers=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_PAGE_NUMBER]
+            ),
+            lists=EntityList([l for l in self.layouts if l.layout_type == LAYOUT_LIST]),
+            figures=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_FIGURE]
+            ),
+            tables=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_TABLE]
+            ),
+            key_values=EntityList(
+                [l for l in self.layouts if l.layout_type == LAYOUT_KEY_VALUE]
+            ),
+        )
 
     @property
     def key_values(self) -> EntityList[KeyValue]:
@@ -214,6 +307,36 @@ class Page(SpatialObject):
         """
         self._signatures = EntityList(signatures)
 
+    @property
+    def layouts(self) -> EntityList[Layout]:
+        """
+        Returns all the :class:`Layout` objects present in the Page.
+
+        :return: List of Layout objects.
+        :rtype: EntityList
+        """
+        return EntityList(sorted(self._layouts, key=lambda c: c.reading_order,))
+
+    @property
+    def layouts(self) -> EntityList[Layout]:
+        """
+        Returns all the :class:`Layout` objects present in the Page.
+
+        :return: List of Layout objects.
+        :rtype: EntityList
+        """
+        return self._layouts
+
+    @layouts.setter
+    def layouts(self, layouts: List[Layout]):
+        """
+        Add layout objects to the Page.
+
+        :param layouts: List of Layout objects.
+        :type layouts: list
+        """
+        self._layouts = EntityList(layouts)
+
     @property
     def expense_documents(self) -> EntityList[ExpenseDocument]:
         """
@@ -246,6 +369,7 @@ class Page(SpatialObject):
                 f"Queries - {len(self.queries)}",
                 f"Signatures - {len(self.signatures)}",
                 f"Expense documents - {len(self.expense_documents)}",
+                f"Layout elements - {len(self.layouts)}",
             ]
         )
 
@@ -286,7 +410,7 @@ class Page(SpatialObject):
         :rtype: EntityList[KeyValue]
         """
         if not self.checkboxes:
-            logging.warning(f"This document does not contain checkboxes")
+            logger.warning(f"This document does not contain checkboxes")
             return []
         else:
             if selected and not_selected:
@@ -327,7 +451,7 @@ class Page(SpatialObject):
             )
 
         if not self.words:
-            logging.warn("Document contains no word entities.")
+            logger.warning("Document contains no word entities.")
             return []
 
         filtered_words = [word for word in self.words if word.text_type == text_type]
@@ -601,11 +725,11 @@ class Page(SpatialObject):
                 lowest_similarity = top_n[-1][1]
 
         if not top_n:
-            logging.warning(
+            logger.warning(
                 f"Query key does not match any existing keys in the document.{os.linesep}{self.keys()}"
             )
 
-        logging.info(f"Query key matched {len(top_n)} key-values in the document.")
+        logger.info(f"Query key matched {len(top_n)} key-values in the document.")
 
         return EntityList([value[0] for value in top_n])
 
@@ -615,6 +739,7 @@ class Page(SpatialObject):
         include_kv: bool = True,
         include_checkboxes: bool = True,
         filepath: str = "Key-Values.csv",
+        sep: str = ",",
     ):
         """
         Export key-value entities and checkboxes in csv format.
@@ -625,29 +750,31 @@ class Page(SpatialObject):
         :type include_checkboxes: bool
         :param filepath: Path to where file is to be stored.
         :type filepath: str
+        :param sep: Separator to be used in the csv file.
+        :type sep: str
         """
         keys = []
         values = []
         if include_kv and not self.key_values:
-            logging.warning("Document does not contain key-values.")
+            logger.warning("Document does not contain key-values.")
         elif include_kv:
             for kv in self.key_values:
                 keys.append(kv.key.__repr__())
                 values.append(kv.value.__repr__())
 
         if include_checkboxes and not self.checkboxes:
-            logging.warning("Document does not contain checkbox elements.")
+            logger.warning("Document does not contain checkbox elements.")
         elif include_checkboxes:
             for kv in self.checkboxes:
                 keys.append(kv.key.__repr__())
                 values.append(kv.value.children[0].status.name)
 
         with open(filepath, "w") as f:
-            f.write(f"Key,Value{os.linesep}")
+            f.write(f"Key{sep}Value{os.linesep}")
             for k, v in zip(keys, values):
-                f.write(f"{k},{v}{os.linesep}")
+                f.write(f"{k}{sep}{v}{os.linesep}")
 
-        logging.info(
+        logger.info(
             f"csv file stored at location {os.path.join(os.getcwd(), filepath)}"
         )
 
@@ -670,7 +797,7 @@ class Page(SpatialObject):
         export_str = []
         index = 1
         if include_kv and not self.key_values:
-            logging.warning("Document does not contain key-values.")
+            logger.warning("Document does not contain key-values.")
         elif include_kv:
             for kv in self.key_values:
                 export_str.append(
@@ -679,7 +806,7 @@ class Page(SpatialObject):
                 index += 1
 
         if include_checkboxes and not self.checkboxes:
-            logging.warning("Document does not contain checkbox elements.")
+            logger.warning("Document does not contain checkbox elements.")
         elif include_checkboxes:
             for kv in self.checkboxes:
                 export_str.append(
@@ -689,7 +816,7 @@ class Page(SpatialObject):
 
         with open(filepath, "w") as text_file:
             text_file.write("".join(export_str))
-        logging.info(
+        logger.info(
             f"txt file stored at location {os.path.join(os.getcwd(),filepath)}"
         )
 
@@ -699,7 +826,7 @@ class Page(SpatialObject):
         :rtype: EntityList[Word]
         """
         if not self.words:
-            logging.warning("Words have not been assigned to this Document object.")
+            logger.warning("Words have not been assigned to this Document object.")
             return []
 
         else:
@@ -722,7 +849,7 @@ class Page(SpatialObject):
         :type filepath: str, required
         """
         if not filepath:
-            logging.error("Filepath required to store excel file.")
+            logger.error("Filepath required to store excel file.")
         workbook = xlsxwriter.Workbook(filepath)
         for table in self.tables:
             workbook = table.to_excel(
@@ -825,7 +952,7 @@ class Page(SpatialObject):
         final_kv = []
         for kv in new_key_values:
             if kv.key:
-                key_words = [deepcopy(word) for word in kv.key.words]
+                key_words = [deepcopy(word) for word in kv.key]
                 key_words[0].text = prefix + key_words[0].text
                 new_kv = deepcopy(kv)
                 new_kv.key = key_words
@@ -892,7 +1019,7 @@ class Page(SpatialObject):
         )
 
         if not word_1_objects:
-            logging.warning(f"{word_1} not found in page")
+            logger.warning(f"{word_1} not found in page")
             return -1, -1, -1, -1
         else:
             word_1_obj = word_1_objects[0]
@@ -907,7 +1034,7 @@ class Page(SpatialObject):
             )
 
             if not word_2_objects:
-                logging.warning(f"{word_2} not found in page")
+                logger.warning(f"{word_2} not found in page")
                 return -1, -1, -1, -1
             else:
                 word_2_obj = word_2_objects[0]
@@ -933,4 +1060,10 @@ class Page(SpatialObject):
         return x1, x2, y1, y2
 
     def visualize(self, *args, **kwargs):
-        return EntityList(self).visualize(*args, **kwargs)
\ No newline at end of file
+        """
+        Returns the object's children in a visualization EntityList object
+
+        :return: Returns an EntityList object
+        :rtype: EntityList
+        """
+        return EntityList(self).visualize(*args, **kwargs)
