@@ -5,15 +5,18 @@ This class contains the associated metadata with the :class:`SelectionElement` e
 bounding box information, selection status, page number, Page ID and confidence of detection.
 """
 
+import uuid
+
 from typing import List
-from textractor.entities.value import Value
+from textractor.data.text_linearization_config import TextLinearizationConfig
 from textractor.entities.word import Word
+from textractor.entities.line import Line
 from textractor.entities.bbox import BoundingBox
 from textractor.data.constants import SELECTED, NOT_SELECTED, SelectionStatus
 from textractor.entities.document_entity import DocumentEntity
 
 
-class SelectionElement(Value):
+class SelectionElement(DocumentEntity):
     """
     To create a new :class:`SelectionElement` object we need the following:
 
@@ -34,12 +37,11 @@ class SelectionElement(Value):
         status: SelectionStatus,
         confidence: float = 0,
     ):
-
         super().__init__(entity_id, bbox)
         self.key_id = None
         self.value_id = None
         self.status = status
-        self.confidence = confidence / 100
+        self._confidence = confidence / 100
         self._page = None
         self._page_id = None
 
@@ -53,11 +55,33 @@ class SelectionElement(Value):
     @property
     def words(self) -> List[Word]:
         """
-        :return: On SelectionElement this should always return an empty list
+        :return: Empty Word list as SelectionElement do not have words
         :rtype: EntityList[Word]
         """
         return []
 
+    def get_text_and_words(
+        self, config: TextLinearizationConfig = TextLinearizationConfig()
+    ):
+        w = Word(
+            entity_id=str(uuid.uuid4()),
+            bbox=self.bbox,
+            text=config.selection_element_selected
+            if self.status == SelectionStatus.SELECTED
+            else config.selection_element_not_selected,
+        )
+        w.is_clickable = True
+        w.line = Line(entity_id=str(uuid.uuid4()), bbox=self.bbox, words=[w])
+
+        words = [w]
+
+        text = w.text
+            
+        for w in words:
+            w.value_id = str(self.id)
+            w.value_bbox = self.bbox
+        return text, words
+
     @property
     def page(self):
         """
