@@ -9,7 +9,7 @@ Represents a single :class:`TableCell:class:` object. The :class:`TableCell` obj
 """
 
 import os
-from typing import List
+from typing import List, Tuple
 
 from textractor.entities.word import Word
 from textractor.exceptions import InputError
@@ -27,6 +27,7 @@ from textractor.data.constants import (
     IS_FOOTER_CELL,
 )
 from textractor.data.constants import TextTypes
+from textractor.utils.text_utils import TextLinearizationConfig, linearize_children
 
 
 class TableCell(DocumentEntity):
@@ -60,16 +61,15 @@ class TableCell(DocumentEntity):
         is_title: bool = False,
         is_footer: bool = False,
         is_summary: bool = False,
-        is_section_title: bool = False
+        is_section_title: bool = False,
     ):
-
         super().__init__(entity_id, bbox)
         self._row_index: int = int(row_index)
         self._col_index: int = int(col_index)
         self._row_span: int = int(row_span)
         self._col_span: int = int(col_span)
         self._words: List[Word] = []
-        self.confidence = confidence / 100
+        self._confidence = confidence / 100
         self._page = None
         self._page_id = None
         self._is_column_header = is_column_header
@@ -90,15 +90,15 @@ class TableCell(DocumentEntity):
     @property
     def is_title(self):
         return self._is_title
-    
+
     @property
     def is_footer(self):
         return self._is_footer
-    
+
     @property
     def is_summary(self):
         return self._is_summary
-    
+
     @property
     def is_section_title(self):
         return self._is_section_title
@@ -199,17 +199,7 @@ class TableCell(DocumentEntity):
         :return words: List of Word objects, each representing a word within the TableCell.
         :rtype: list
         """
-        return EntityList(self._words)
-
-    @words.setter
-    def words(self, words: List[Word]):
-        """
-        Add Word objects to the :class:`TableCell`.
-
-        :param words: List of Word objects, each representing a word within the TableCell. No specific ordering is assumed as it is ordered internally.
-        :type words: list
-        """
-        self._words = words
+        return EntityList(self.get_text_and_words()[1])
 
     @property
     def checkboxes(self):
@@ -219,7 +209,6 @@ class TableCell(DocumentEntity):
                 output.append(child)
         return output
 
-
     @property
     def text(self) -> str:
         """Returns the text in the cell as one space-separated string
@@ -227,7 +216,18 @@ class TableCell(DocumentEntity):
         :return: Text in the cell
         :rtype: str
         """
-        return " ".join([" ".join([str(c) for c in self.checkboxes]), " ".join([w.text for w in self.words])])
+        return self.get_text()
+
+    def get_text_and_words(
+        self, config: TextLinearizationConfig = TextLinearizationConfig()
+    ) -> Tuple[str, List]:
+        """Returns the text in the cell as one space-separated string
+
+        :return: Text in the cell
+        :rtype: Tuple[str, List]
+        """
+        text, words = linearize_children(self.children, config=config, no_new_lines=True)
+        return text, words
 
     @property
     def table_id(self):
@@ -324,7 +324,7 @@ class TableCell(DocumentEntity):
         if self.metadata.get(IS_MERGED_CELL, False):
             entities = {
                 (cell.row_index, cell.col_index): sorted(
-                    cell.words + cell.children, key=lambda x: (x.bbox.x, x.bbox.y)
+                    cell.words, key=lambda x: (x.bbox.x, x.bbox.y)
                 )
                 for cell in sorted(
                     self.siblings, key=lambda x: (x.row_index, x.col_index)
@@ -345,7 +345,7 @@ class TableCell(DocumentEntity):
             entity_repr = "".join(entity_repr)
 
         else:
-            entities = self.words + self.children
+            entities = self.words
             entity_repr = " ".join([entity.__repr__() for entity in entities])
 
         entity_string = f"<Cell: ({self.row_index},{self.col_index}), Span: ({self.row_span}, {self.col_span}), Column Header: { self.is_column_header}, "
