@@ -53,3 +53,13 @@ class UnhandledCaseException(Exception):
     """Raised when no statement matched the condition"""
 
     pass
+
+class UnsupportedDocumentException(Exception):
+    """Raised by the Textract API when the document could not be processed"""
+
+    pass
+
+class InvalidS3ObjectException(Exception):
+    """Raised by the Textract API when an S3 object could not be accessed"""
+    
+    pass
