@@ -34,15 +34,6 @@ from textractcaller import (
     QueriesConfig,
 )
 from textractcaller.t_call import Textract_Call_Mode, Textract_API, get_full_json
-
-try:
-    from pdf2image import convert_from_bytes, convert_from_path
-
-    IS_PDF2IMAGE_INSTALLED = True
-except ImportError:
-    IS_PDF2IMAGE_INSTALLED = False
-    logging.info("pdf2image is not installed, client-side PDF rasterizing is disabled")
-
 from textractor.data.constants import (
     TextractAPI,
     TextractFeatures,
@@ -51,14 +42,28 @@ from textractor.entities.document import Document
 from textractor.entities.lazy_document import LazyDocument
 from textractor.parsers import response_parser
 from textractor.utils.s3_utils import upload_to_s3, s3_path_to_bucket_and_prefix
+from textractor.utils.pdf_utils import rasterize_pdf
 from textractor.exceptions import (
     InputError,
-    RegionMismatchError,
     IncorrectMethodException,
     MissingDependencyException,
     UnhandledCaseException,
+    UnsupportedDocumentException,
+    InvalidS3ObjectException,
 )
 
+logger = logging.getLogger(__name__)
+
+try:
+    try:
+        import pypdfium2
+    except:
+        import pdf2image
+
+    IS_PDF_RENDERING_ENABLED = True
+except ImportError:
+    IS_PDF_RENDERING_ENABLED = False
+    logger.info("pypdfium2 and pdf2image are both not installed, client-side PDF rasterizing is disabled")
 
 class Textractor:
     """
@@ -90,12 +95,18 @@ class Textractor:
             self.session = boto3.session.Session(profile_name=self.profile_name)
         elif self.region_name is not None:
             self.session = boto3.session.Session(region_name=self.region_name)
+        elif os.environ.get("AWS_REGION") or os.environ.get("AWS_DEFAULT_REGION"):
+            # We support both AWS_REGION and AWS_DEFAULT_REGION, with AWS_REGION having precedence.
+            self.region_name = os.environ.get("AWS_REGION") or os.environ.get("AWS_DEFAULT_REGION")
+            self.session = boto3.session.Session(region_name=self.region_name)
         else:
             raise InputError(
                 "Unable to initiate Textractor. Either profile_name or region requires an input parameter."
             )
         if self.region_name is not None:
-            self.textract_client = self.session.client("textract", region_name=self.region_name)
+            self.textract_client = self.session.client(
+                "textract", region_name=self.region_name
+            )
         else:
             self.textract_client = self.session.client("textract")
         self.s3_client = self.session.client("s3")
@@ -116,28 +127,27 @@ class Textractor:
             bucket = edit_filepath.split("/")[0]
             key = edit_filepath[edit_filepath.index("/") + 1 :]
 
-            s3_client = (
-                boto3.session.Session(profile_name=self.profile_name).client("s3")
-                if self.profile_name is not None
-                else boto3.session.Session(region_name=self.region_name).client("s3")
-            )
-            file_obj = s3_client.get_object(Bucket=bucket, Key=key).get("Body").read()
+            file_obj = self.s3_client.get_object(Bucket=bucket, Key=key).get("Body").read()
             if filepath.lower().endswith(".pdf"):
-                if IS_PDF2IMAGE_INSTALLED:
-                    images = convert_from_bytes(bytearray(file_obj))
+                if IS_PDF_RENDERING_ENABLED:
+                    images = rasterize_pdf(file_obj)
                 else:
-                    raise MissingDependencyException("pdf2image is not installed. If you do not plan on using visualizations you can skip image generation using save_image=False in your function call.")
+                    raise MissingDependencyException(
+                        "pdf2image is not installed. If you do not plan on using visualizations you can skip image generation using save_image=False in your function call."
+                    )
             else:
                 images = [Image.open(io.BytesIO(bytearray(file_obj)))]
 
         else:
             if filepath.lower().endswith(".pdf"):
-                if IS_PDF2IMAGE_INSTALLED:
-                    images = convert_from_path(filepath)
+                if IS_PDF_RENDERING_ENABLED:
+                    images = rasterize_pdf(filepath)
                 else:
-                    raise MissingDependencyException("pdf2image is not installed. If you do not plan on using visualizations you can skip image generation using save_image=False in your function call.")
+                    raise MissingDependencyException(
+                        "pdf2image is not installed. If you do not plan on using visualizations you can skip image generation using save_image=False in your function call."
+                    )
             else:
-                images = [Image.open(open(filepath, "rb"))]
+                images = [Image.open(filepath)]
 
         if not images:
             raise UnhandledCaseException(f"Could not get any images from {filepath}")
@@ -145,7 +155,7 @@ class Textractor:
         return images
 
     def detect_document_text(
-        self, file_source, s3_output_path: str = "", save_image: bool = True
+        self, file_source, save_image: bool = True
     ) -> Document:
         """
         Make a call to the SYNC DetectDocumentText API, implicitly parses the response and produces a :class:`Document` object.
@@ -153,8 +163,6 @@ class Textractor:
 
         :param file_source: Path to a file stored locally, on an S3 bucket or PIL Image
         :type file_source: str or PIL.Image, required
-        :param s3_output_path: S3 path to store the output.
-        :type s3_output_path: str, optional
         :param save_image: Flag to indicate if document images are to be stored within the Document object. This is optional
                             and necessary only if the customer wants to visualize bounding boxes for their document entities.
         :type save_image: bool
@@ -170,21 +178,24 @@ class Textractor:
             )
 
         elif isinstance(file_source, str):
-            logging.debug("Filepath given.")
-            images = self._get_document_images_from_path(file_source)
-            if len(images) > 1:
-                raise IncorrectMethodException(
-                    "Input contains more than 1 page. Call start_document_text_detection instead."
-                )
-            file_source = _image_to_byte_array(images[0])
+            logger.debug("Filepath given.")
+            if not save_image and file_source.lower().endswith(".pdf"):
+                images = []
+            else:
+                images = self._get_document_images_from_path(file_source)
+                if len(images) > 1:
+                    raise IncorrectMethodException(
+                        "Input contains more than 1 page. Call start_document_analysis() instead."
+                    )
+                file_source = _image_to_byte_array(images[0])    
 
         elif isinstance(file_source, Image.Image):
-            logging.debug("PIL Image given.")
+            logger.debug("PIL Image given.")
             images = [file_source]
             file_source = _image_to_byte_array(file_source)
 
         elif isinstance(file_source, list) and isinstance(file_source[0], Image.Image):
-            logging.debug("List of PIL Image given.")
+            logger.debug("List of PIL Image given.")
             images = deepcopy(file_source)
             file_source = _image_to_byte_array(images[0])
 
@@ -192,18 +203,12 @@ class Textractor:
             images = []
             raise InputError("Input file_source format not supported.")
 
-        if not s3_output_path:
-            output_config = None
-        else:
-            bucket, prefix = s3_path_to_bucket_and_prefix(s3_output_path)
-            output_config = OutputConfig(s3_bucket=bucket, s3_prefix=prefix)
-
         try:
             response = call_textract(
                 input_document=file_source,
                 features=[],
                 queries_config=None,  # not supported yet
-                output_config=output_config,
+                output_config=None,
                 kms_key_id=self.kms_key_id,
                 job_tag="",
                 notification_channel=None,  # not supported yet
@@ -216,8 +221,12 @@ class Textractor:
             )
         except Exception as exception:
             if exception.__class__.__name__ == "InvalidS3ObjectException":
-                raise RegionMismatchError(
-                    "Region passed in the profile_name and S3 bucket do not match. Ensure the regions are the same."
+                raise InvalidS3ObjectException(
+                    "Textract returned InvalidS3ObjectException. Ensure that the s3 path is correct and that both the Textract API and the bucket are in the same region."
+                )
+            elif exception.__class__.__name__ == "UnsupportedDocumentException":
+                raise UnsupportedDocumentException(
+                    "Textract returned UnsupportedDocumentException, if file_source is a PDF, make sure that it only has one page or use start_document_text_detection. If your file_source is an image, make sure that it is not larger than 5MB."
                 )
             raise exception
 
@@ -236,7 +245,7 @@ class Textractor:
         client_request_token: str = "",
         job_tag: str = "",
         save_image: bool = True,
-    ):
+    ) -> LazyDocument:
         """
         Make a call to the ASYNC StartDocumentTextDetection API.
 
@@ -253,18 +262,17 @@ class Textractor:
         :type client_request_token: str, optional
         :param job_tag: An identifier that you specify that's included in the completion notification published to the Amazon SNS topic.
         :type job_tag: str, optional
+        :param save_image: Flag to indicate if document images are to be stored within the Document object. This is optional
+                            and necessary only if the customer wants to visualize bounding boxes for their document entities.
+        :type save_image: bool
 
-        :return: Returns a job id which can be used to fetch the results
-        :rtype: str
+        :return: Lazy-loaded Document object
+        :rtype: LazyDocument
         """
 
         original_file_source = file_source
 
-        if not (
-            isinstance(file_source, str)
-            or isinstance(file_source, bytes)
-            or isinstance(file_source, Image.Image)
-        ):
+        if not isinstance(file_source, (str, bytes, Image.Image)):
             raise InputError(
                 f"file_source needs to be of type str, bytes or PIL Image, not {type(file_source)}"
             )
@@ -274,7 +282,7 @@ class Textractor:
             # Check if the user has given us a bucket to upload to
             if not s3_upload_path:
                 raise InputError(
-                    f"For files not in S3, an S3 upload path must be provided"
+                    "For files not in S3, an S3 upload path must be provided"
                 )
 
             s3_file_path = os.path.join(s3_upload_path, str(uuid.uuid4()))
@@ -304,8 +312,8 @@ class Textractor:
             )
         except Exception as exception:
             if exception.__class__.__name__ == "InvalidS3ObjectException":
-                raise RegionMismatchError(
-                    "Region passed in the profile_name and S3 bucket do not match. Ensure the regions are the same."
+                raise InvalidS3ObjectException(
+                    "Textract returned InvalidS3ObjectException. Ensure that the s3 path is correct and that both the Textract API and the bucket are in the same region."
                 )
             raise exception
 
@@ -326,6 +334,7 @@ class Textractor:
             response["JobId"],
             TextractAPI.DETECT_TEXT,
             textract_client=self.textract_client,
+            s3_client=self.s3_client,
             images=images,
         )
 
@@ -334,7 +343,6 @@ class Textractor:
         file_source,
         features,
         queries: Union[QueriesConfig, List[Query], List[str]] = None,
-        s3_output_path: str = "",
         save_image: bool = True,
     ) -> Document:
         """
@@ -346,9 +354,7 @@ class Textractor:
         :param features: List of TextractFeatures to be extracted from the Document by the TextractAPI
         :type features: list, required
         :param queries: Queries to run on the document
-        :type features: Union[QueriesConfig, List[Query], List[str]]
-        :param s3_output_path: Prefix to store the output on the S3 bucket (passed as param to Textractor).
-        :type s3_output_path: str, optional
+        :type queries: Union[QueriesConfig, List[Query], List[str]]
         :param save_image: Flag to indicate if document images are to be stored within the Document object. This is optional
                             and necessary only if the customer wants to visualize bounding boxes for their document entities.
         :type save_image: bool
@@ -363,21 +369,23 @@ class Textractor:
             )
 
         elif isinstance(file_source, str):
-            logging.debug("Filepath given.")
-            images = self._get_document_images_from_path(file_source)
-            if len(images) > 1:
-                raise IncorrectMethodException(
-                    "Input contains more than 1 page. Call start_document_analysis() instead."
-                )
-            file_source = _image_to_byte_array(images[0])
-
+            logger.debug("Filepath given.")
+            if not save_image and file_source.lower().endswith(".pdf"):
+                images = []
+            else:
+                images = self._get_document_images_from_path(file_source)
+                if len(images) > 1:
+                    raise IncorrectMethodException(
+                        "Input contains more than 1 page. Call start_document_analysis() instead."
+                    )
+                file_source = _image_to_byte_array(images[0])    
         elif isinstance(file_source, Image.Image):
-            logging.debug("PIL Image given.")
+            logger.debug("PIL Image given.")
             images = [file_source]
             file_source = _image_to_byte_array(file_source)
 
         elif isinstance(file_source, list) and isinstance(file_source[0], Image.Image):
-            logging.debug("List of PIL Image given.")
+            logger.debug("List of PIL Image given.")
             images = deepcopy(file_source)
             file_source = _image_to_byte_array(images[0])
 
@@ -385,12 +393,6 @@ class Textractor:
             images = []
             raise InputError("Input file_source format not supported.")
 
-        if not s3_output_path:
-            output_config = None
-        else:
-            bucket, prefix = s3_path_to_bucket_and_prefix(s3_output_path)
-            output_config = OutputConfig(s3_bucket=bucket, s3_prefix=prefix)
-
         if not isinstance(features, list):
             features = [features]
 
@@ -420,7 +422,7 @@ class Textractor:
                 input_document=file_source,
                 features=features,
                 queries_config=queries,  # not supported yet
-                output_config=output_config,
+                output_config=None,
                 kms_key_id=self.kms_key_id,
                 job_tag="",
                 notification_channel=None,  # not supported yet
@@ -433,8 +435,12 @@ class Textractor:
             )
         except Exception as exception:
             if exception.__class__.__name__ == "InvalidS3ObjectException":
-                raise RegionMismatchError(
-                    "Region passed in the profile_name and S3 bucket do not match. Ensure the regions are the same."
+                raise InvalidS3ObjectException(
+                    "Textract returned InvalidS3ObjectException. Ensure that the s3 path is correct and that both the Textract API and the bucket are in the same region."
+                )
+            elif exception.__class__.__name__ == "UnsupportedDocumentException":
+                raise UnsupportedDocumentException(
+                    "Textract returned an UnsupportedDocumentException, if file_source is a PDF, make sure that it only has one page or use start_document_analysis. If your file_source is an image, make sure that it is not larger than 5MB."
                 )
             raise exception
 
@@ -486,11 +492,7 @@ class Textractor:
 
         original_file_source = file_source
 
-        if not (
-            isinstance(file_source, str)
-            or isinstance(file_source, bytes)
-            or isinstance(file_source, Image.Image)
-        ):
+        if not isinstance(file_source, (str, bytes, Image.Image)):
             raise InputError(
                 f"file_source needs to be of type str, bytes or PIL Image, not {type(file_source)}"
             )
@@ -554,8 +556,8 @@ class Textractor:
             )
         except Exception as exception:
             if exception.__class__.__name__ == "InvalidS3ObjectException":
-                raise RegionMismatchError(
-                    "Region passed in the profile_name and S3 bucket do not match. Ensure the regions are the same."
+                raise InvalidS3ObjectException(
+                    "Textract returned InvalidS3ObjectException. Ensure that the s3 path is correct and that both the Textract API and the bucket are in the same region."
                 )
             raise exception
 
@@ -576,7 +578,9 @@ class Textractor:
             response["JobId"],
             TextractAPI.ANALYZE,
             textract_client=self.textract_client,
+            s3_client=self.s3_client,
             images=images,
+            output_config=output_config,
         )
 
     def analyze_id(
@@ -593,19 +597,19 @@ class Textractor:
         :param save_image: Saves the images in the returned Document object for visualizing the results, defaults to False
         :type save_image: bool, optional
         :raises InputError: Raised when the file_source could not be parsed
-        :raises RegionMismatchError: Raised when the S3 object passed as file source is in a region that does not match the one used to create the Textractor object.
+        :raises InvalidS3ObjectException: Raised when the S3 object passed as file source is in a region that does not match the one used to create the Textractor object.
         :raises exception: Raised when the Textract call fails
         :return: Document
         :rtype: Document
         """
         if isinstance(file_source, str):
-            logging.debug("Filepath given.")
+            logger.debug("Filepath given.")
             images = self._get_document_images_from_path(file_source)
         elif isinstance(file_source, Image.Image):
-            logging.debug("PIL Image given.")
+            logger.debug("PIL Image given.")
             images = [file_source]
         elif isinstance(file_source, list) and isinstance(file_source[0], Image.Image):
-            logging.debug("List of PIL Image given.")
+            logger.debug("List of PIL Image given.")
             # FIXME: Is this needed?
             images = deepcopy(file_source)
         else:
@@ -621,8 +625,8 @@ class Textractor:
             )
         except Exception as exception:
             if exception.__class__.__name__ == "InvalidS3ObjectException":
-                raise RegionMismatchError(
-                    "Region passed in the profile_name and S3 bucket do not match. Ensure the regions are the same."
+                raise InvalidS3ObjectException(
+                    "Textract returned InvalidS3ObjectException. Ensure that the s3 path is correct and that both the Textract API and the bucket are in the same region."
                 )
             raise exception
 
@@ -636,7 +640,6 @@ class Textractor:
     def analyze_expense(
         self,
         file_source: Union[str, List[Image.Image], List[str]],
-        s3_output_path: str = "",
         save_image: bool = True,
     ):
         """Make a call to the SYNC AnalyzeExpense API, implicitly parses the response and produces a :class:`Document` object.
@@ -644,13 +647,11 @@ class Textractor:
 
         :param file_source: Path to a file stored locally, on an S3 bucket or PIL Image
         :type file_source: Union[str, List[Image.Image], List[str]]
-        :param s3_output_path: S3 output path. When used the job output is save to the given S3 path, defaults to ""
-        :type s3_output_path: str, optional
         :param save_image: Whether to keep the file source as PIL Images inside the returned Document object, defaults to False
         :type save_image: bool, optional
         :raises IncorrectMethodException: Raised when the file source type is incompatible with the Textract API being called
         :raises InputError: Raised when the file source type is invalid
-        :raises RegionMismatchError: Raised when the file source region is different the API region.
+        :raises InvalidS3ObjectException: Raised when the file source region is different the API region.
         :raises exception: Raised if the Textract API call fails
         :return: Document
         :rtype: Document
@@ -661,7 +662,7 @@ class Textractor:
             )
 
         elif isinstance(file_source, str):
-            logging.debug("Filepath given.")
+            logger.debug("Filepath given.")
             images = self._get_document_images_from_path(file_source)
             if len(images) > 1:
                 raise IncorrectMethodException(
@@ -670,12 +671,12 @@ class Textractor:
             file_source = _image_to_byte_array(images[0])
 
         elif isinstance(file_source, Image.Image):
-            logging.debug("PIL Image given.")
+            logger.debug("PIL Image given.")
             images = [file_source.copy()]
             file_source = _image_to_byte_array(file_source)
 
         elif isinstance(file_source, list) and isinstance(file_source[0], Image.Image):
-            logging.debug("List of PIL Image given.")
+            logger.debug("List of PIL Image given.")
             images = deepcopy(file_source)
             file_source = _image_to_byte_array(images[0])
 
@@ -683,11 +684,7 @@ class Textractor:
             images = []
             raise InputError("Input file_source format not supported.")
 
-        if not s3_output_path:
-            output_config = None
-        else:
-            bucket, prefix = s3_path_to_bucket_and_prefix(s3_output_path)
-            output_config = OutputConfig(s3_bucket=bucket, s3_prefix=prefix)
+        output_config = None
 
         try:
             response = call_textract_expense(
@@ -704,8 +701,8 @@ class Textractor:
             )
         except Exception as exception:
             if exception.__class__.__name__ == "InvalidS3ObjectException":
-                raise RegionMismatchError(
-                    "Region passed in the profile_name and S3 bucket do not match. Ensure the regions are the same."
+                raise InvalidS3ObjectException(
+                    "Textract returned InvalidS3ObjectException. Ensure that the s3 path is correct and that both the Textract API and the bucket are in the same region."
                 )
             raise exception
 
@@ -745,19 +742,15 @@ class Textractor:
                             and necessary only if the customer wants to visualize bounding boxes for their document entities.
         :type save_image: bool
         :raises InputError: Raised when the file source type is invalid
-        :raises RegionMismatchError: Raised when the file source region is different the API region.
+        :raises InvalidS3ObjectException: Raised when the file source region is different the API region.
         :raises exception: Raised if the Textract API call fails
         :return: Lazy-loaded Document object
         :rtype: LazyDocument
         """
 
         original_file_source = file_source
-        
-        if not (
-            isinstance(file_source, str)
-            or isinstance(file_source, bytes)
-            or isinstance(file_source, Image.Image)
-        ):
+
+        if not isinstance(file_source, (str, bytes, Image.Image)):
             raise InputError(
                 f"file_source needs to be of type str, bytes or PIL Image, not {type(file_source)}"
             )
@@ -794,8 +787,8 @@ class Textractor:
             )
         except Exception as exception:
             if exception.__class__.__name__ == "InvalidS3ObjectException":
-                raise RegionMismatchError(
-                    "Region passed in the profile_name and S3 bucket do not match. Ensure the regions are the same."
+                raise InvalidS3ObjectException(
+                    "Textract returned InvalidS3ObjectException. Ensure that the s3 path is correct and that both the Textract API and the bucket are in the same region."
                 )
             raise exception
 
@@ -816,11 +809,20 @@ class Textractor:
             response["JobId"],
             TextractAPI.EXPENSE,
             textract_client=self.textract_client,
+            s3_client=self.s3_client,
             images=images,
         )
 
-    def get_result(self, job_id: str, api: Union[TextractAPI, Textract_API]):
-        """ """
+    def get_result(
+        self, job_id: str, api: Union[TextractAPI, Textract_API]
+    ) -> Document:
+        """
+        Retrieves Textract API output for a given job id.
+        :param job_id: Textract API JobID
+        :type job_id: str, required
+        :return: Returns a Document object
+        :rtype: Document
+        """
 
         response = get_full_json(
             job_id,
@@ -846,6 +848,7 @@ def _image_to_byte_array(image: Image) -> bytes:
     :rtype: bytes
     """
     img_byte_arr = io.BytesIO()
-    image.save(img_byte_arr, format="PNG")
+    # We set quality to 95 and subsampling to 0 because the pillow defaults are very low resolution
+    image.convert("RGB").save(img_byte_arr, format="JPEG", quality=95, subsampling=0)
     img_byte_arr = img_byte_arr.getvalue()
     return img_byte_arr
